<?php

namespace App\Services\Data;

use App\Models\JuxingAdAccount;
use App\Support\EmailQueue;
use App\Support\Log;
use App\Support\RedisModel;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redis;
use kwaiSDK\JuXing;

class SupplementOrderService
{
    public static function supplementOrderStop($supplementOrderList, $supplementOrderType) {
        # 处理订单列表信息(按广告主账户分类)
        $request = [];
        $supplementOrderList = json_decode($supplementOrderList, 1);
        foreach($supplementOrderList as $supplementOrder) {
            $advertiserId = $supplementOrder['advertiser_id'] ?? '';
            $supplementOrderId = $supplementOrder['supplement_order_id'] ?? '';
            $supplementOrderId = intval($supplementOrderId);
            if(empty($advertiserId) || empty($supplementOrderId)) {
                Log::error('请求数据格式不合法', [$supplementOrderList], 'supplementOrder');
                return ['请求失败', 1102];
            }

            $request[$advertiserId][] = $supplementOrderId;
        }

        $result = [
            'stat' => ['success' => 0, 'fail' => 0, 'total' => 0],
            'data' => []
        ];
        foreach ($request as $advertiserId => $supplementOrderIdList) {
            # 避免一次请求订单数太多导致失败，将订单分批次处理
            $supplementOrderIdArr = array_chunk($supplementOrderIdList, 100);
            foreach($supplementOrderIdArr as $newSupplementOrderIdList) {
                list($data, $message) = self::supplementOrderStopRequest($advertiserId, $supplementOrderType, $newSupplementOrderIdList);
            }

            $result['stat']['success'] += $data['success']['num'];
            $result['stat']['total']   += $data['total'];
            $result['stat']['fail']    += $data['fail']['num'];
            if($data['success']['num'] > 0) {
                // 将广告主账户ID放入更新队列中或者直接更新系统数据
                self::syncFlowOrder($advertiserId);
            }
        }

        return [$result, 0];
    }

    public static function syncFlowOrder($advertiserId) {
        $stTime = date('Y-m-d', strtotime('-90 days'));
        $enTime = date('Y-m-d');
        # 获取时间段数组
        $timeRangeArr = getTimeRangeArr($stTime, $enTime);
        foreach ($timeRangeArr as $timeRange) {
            # 5分钟内不重复更新
            $redisKey = 'juxingFlowStarVideoSyncAdvidListLck-'.$advertiserId;
            $redisValue = RedisModel::get($redisKey);
            if(!empty($redisValue)) {
                continue;
            }

            # 插入队列
            $advertiserItem = [
                'advertiserId' => $advertiserId,
                'timeRange'    => $timeRange
            ];
            RedisModel::lPush(JuxingAdAccount::JUXING_FLOW_STAR_VIDEO_SYNC_ADVID_LIST, json_encode($advertiserItem));

            # 插入队列
            RedisModel::lPush(JuxingAdAccount::JUXING_FLOW_QTASK_VIDEO_SYNC_ADVID_LIST, json_encode($advertiserItem));

            RedisModel::set($redisKey, 1);
            RedisModel::expire($redisValue, 300);
        }
    }

    public static function supplementOrderStopRequest($advertiserId, $supplementOrderType, $supplementOrderIdList, $retry=0) {
        $accessToken = JuxingAdAccount::getAccessToken($advertiserId);
        $juxing = new JuXing($accessToken);

        $param = [
            'advertiser_id' => intval($advertiserId),
            'supplement_order_type' => intval($supplementOrderType),
            'supplement_order_ids' => $supplementOrderIdList
        ];
        $rst = $juxing->supplementOrderV2()->stop($param);
        if($rst === false && $retry < 3) {
            $retry++;
            return self::supplementOrderStopRequest($advertiserId, $supplementOrderType, $supplementOrderIdList, $retry);
        }
        Log::info('助推订单停投', ['param' => $param, 'ret' => $rst, 'operator_id' => Auth::id()], 'supplementOrder');
        $data = [
            'success' => ['list' => [], 'num' => 0], 'fail' => ['list' => [], 'num' => 0], 'total' => count($supplementOrderIdList)
        ];
        if(isset($rst['code']) && $rst['code'] == 0) {# 请求成功
            $supplementOrder = $rst['data']['stop_supplement_orders'] ?? [];
            foreach($supplementOrder as $order) {
                if(isset($order['result']) && $order['result']) {
                    $data['success']['list'][] = $order['supplement_order_id'];
                    $data['success']['num']++;
                } else {
                    $data['fail']['list'][] = $order['supplement_order_id'];
                    $data['fail']['num']++;
                }
            }

        } else {
            if($retry < 3) {
                $retry++;
                return self::supplementOrderStopRequest($advertiserId, $supplementOrderType, $supplementOrderIdList, $retry);
            }
            $data['fail']['list'] = $supplementOrderIdList;
            $data['fail']['num'] = count($supplementOrderIdList);
            EmailQueue::rPush('助推订单停投返回异常', json_encode([
                'advertiser_id' => $advertiserId,
                'supplement_order_id_list' => $supplementOrderIdList,
                'supplement_order_type' => $supplementOrderType,
                'response' => $rst
            ], 256), ['song.shen@kuxuan-inc.com'], '聚星系统');
        }

        return [$data, $rst['message']];
    }

    public static function supplementOrderUpdatePromotionEndTime($advertiserId, $supplementOrderId, $promotionEndTime, $retry = 0) {
        $accessToken = JuxingAdAccount::getAccessToken($advertiserId);
        $juxing = new JuXing($accessToken);
        $realPromotionEndTime = date("Y-m-d H:i:s", strtotime($promotionEndTime)).'.'.rand(100, 999);
        $param = [
            'advertiser_id' => intval($advertiserId),
            'supplement_order_id' => intval($supplementOrderId),
            'promotion_end_time' => $realPromotionEndTime
        ];

        $rst = $juxing->supplementOrderV2()->promotionEndtimeUpdate($param);
        if($rst === false && $retry < 3) {
            $retry++;
            return self::supplementOrderUpdatePromotionEndTime($advertiserId, $supplementOrderId, $promotionEndTime, $retry);
        }
        Log::info('助推订单修改推广结束时间', ['param' => $param, 'ret' => $rst, 'operator_id' => Auth::id()], 'supplementOrder');
        if(isset($rst['code']) && $rst['code'] == 0) {# 请求成功
            // 将广告主账户ID放入更新队列中或者直接更新系统数据
            self::syncFlowOrder($advertiserId);
            return ['', 0, null];
        } else {
            if($retry < 3) {
                $retry++;
                return self::supplementOrderUpdatePromotionEndTime($advertiserId, $supplementOrderId, $promotionEndTime, $retry);
            }
            EmailQueue::rPush('助推订单修改推广结束时间返回异常', json_encode([
                'advertiser_id' => $advertiserId,
                'supplement_order_id' => $supplementOrderId,
                'promotion_end_time' => $realPromotionEndTime,
                'response' => $rst
            ], 256), ['song.shen@kuxuan-inc.com'], '聚星系统');
            return ['请求失败', 501, $rst['message']];
        }
    }

    public static function supplementOrderUpgradeAmount($advertiserId, $supplementOrderId, $payMethod, $amount, $retry = 0) {
        $accessToken = JuxingAdAccount::getAccessToken($advertiserId);
        $juxing = new JuXing($accessToken);
        $param = [
            'advertiser_id' => intval($advertiserId),
            'supplement_order_id' => intval($supplementOrderId),
            'pay_method' => $payMethod,
            'increased_amount' => intval($amount * 1000), // 单位厘
        ];

        $rst = $juxing->supplementOrderV2()->amountUpgrade($param);
        if($rst === false && $retry < 3) {
            $retry++;
            return self::supplementOrderUpgradeAmount($advertiserId, $supplementOrderId, $payMethod, $amount, $retry);
        }
        Log::info('助推订单升单', ['param' => $param, 'ret' => $rst, 'operator_id' => Auth::id()], 'supplementOrder');
        if(isset($rst['code']) && $rst['code'] == 0) {# 请求成功
            // 将广告主账户ID放入更新队列中或者直接更新系统数据
            self::syncFlowOrder($advertiserId);
            return ['', 0, null];
        } else {
            if($retry < 3) {
                $retry++;
                return self::supplementOrderUpgradeAmount($advertiserId, $supplementOrderId, $payMethod, $amount, $retry);
            }
            EmailQueue::rPush('助推订单升单请求失败', json_encode([
                'advertiser_id' => $advertiserId,
                'supplement_order_id' => $supplementOrderId,
                'pay_method' => $payMethod,
                'amount' => $amount,
                'response' => $rst
            ], 256), ['song.shen@kuxuan-inc.com'], '聚星系统');
            return ['请求失败', 501, $rst['message']];
        }
    }

    public static function supplementOrderUpdateUnitPrice($advertiserId, $supplementOrderId, $price, $retry = 0) {
        $accessToken = JuxingAdAccount::getAccessToken($advertiserId);
        $juxing = new JuXing($accessToken);
        $param = [
            'advertiser_id' => intval($advertiserId),
            'supplement_order_id' => intval($supplementOrderId),
            'unit_price' => intval($price * 1000)// 单位厘
        ];

        $rst = $juxing->supplementOrderV2()->unitPriceUpdate($param);
        if($rst === false && $retry < 3) {
            $retry++;
            return self::supplementOrderUpdateUnitPrice($advertiserId, $supplementOrderId, $price, $retry);
        }
        Log::info('助推订单修改出价', ['param' => $param, 'ret' => $rst, 'operator_id' => Auth::id()], 'supplementOrder');
        if(isset($rst['code']) && $rst['code'] == 0) {# 请求成功
            // 将广告主账户ID放入更新队列中或者直接更新系统数据
            self::syncFlowOrder($advertiserId);
            return ['', 0, null];
        } else {
            if($retry < 3) {
                $retry++;
                return self::supplementOrderUpdateUnitPrice($advertiserId, $supplementOrderId, $price, $retry);
            }

            EmailQueue::rPush('助推订单修改出价请求失败', json_encode([
                'advertiser_id' => $advertiserId,
                'supplement_order_id' => $supplementOrderId,
                'price' => $price,
                'response' => $rst
            ], 256), ['song.shen@kuxuan-inc.com'], '聚星系统');
            return ['请求失败', 501, $rst['message']];
        }
    }
}
